<?php
/*
Plugin Name: FriendFeed Comments
Version: 1.4
Plugin URI: http://blog.slaven.net.au/wordpress-plugins/friendfeed-comments-wordpress-plugin/
Description: Places the comments from friend feed on each post from your blog under the normal comments section on the post page.
Author: Glenn Slaven
Author URI: http://blog.slaven.net.au/

	Copyright 2008 Glenn Slaven  (email : gdalziel@gmail.com)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
if (!class_exists('FriendFeed')) {
	require_once("friendfeed.php");
}

class FriendFeedPostComments
{
	var $options_key = 'friendfeed_options';
	var $discussions_key = 'friendfeed_discussions';
	var $max_posts_to_load = 150;
	var $page_size = 30;
	var $is_debug = false;
	
	function FriendFeedPostComments()
	{
		add_action('admin_menu', array(&$this, '_create_options_menu_item'));
		add_action('update_friendfeedcomments_hook', array(&$this, 'update_friendfeedcomments'));
		add_action('wp_head', array(&$this, 'add_page_header'), 1);
		
		if (method_exists($this, '_install')) { register_activation_hook(__FILE__, array(&$this, '_install')); }
		if (method_exists($this, '_uninstall')) { register_deactivation_hook(__FILE__, array(&$this, '_uninstall')); }
	}
	
	//Callback to create the Settings menu option
	function _create_options_menu_item(){
		if (function_exists('add_options_page')) { add_options_page("FriendFeed Comments", "FriendFeed Comments", 9, __FILE__, array(&$this, 'options_page')); }
		add_action('admin_notices', array(&$this, '_admin_notices'));
	}
	
	//Display admin notices if needed
	function _admin_notices() {
		$options = get_option($this->options_key);
		if ((!$options || !$options['nickname']) && !$_POST['ff_nickname'])
		{
			echo "<div class='updated' style='background-color:#f66;'><p><a href=\"options-general.php?page=friendfeed-comments/friendfeed-comments.php\">FriendFeed Comments</a> does not have a nickname set.  You need to enter your FriendFeed nickname for the comments to be loaded.</p></div>";
		}
	}
	
	//Check if the debug flag is set
	function is_debug() {
		if ($_REQUEST['ffdebugflag']) {
			$this->is_debug = true;
		}
		
		return $this->is_debug;
	}
	
	//Callback to setup the plugin
	function _install() {
		wp_schedule_event( time(), 'hourly', 'update_friendfeedcomments_hook' );
		$options = get_option($this->options_key);
		if (!$options) {
				$options['nickname'] = '';
				$options['servicename'] = 'blog';
				$options['showcomments'] = false;
				$options['headline'] = false;
				$options['showform'] = true;
				$options['usestyles'] = true;
			update_option($this->options_key, $options);
		}
		
	}
	
	//Callback to unstall the plugin
	function _uninstall() {
		wp_clear_scheduled_hook('update_friendfeedcomments_hook');
	}
	
	//Format the title into a normalised string that can be used as a hash key
	function format_title($title) {
		return md5(urlencode(ereg_replace("[/,\s\:\"]", "", strtolower(str_replace(urldecode('&#8217;'), '', $title)))));
	}
	
	//Debug output
	function printr($string) {
		print "<pre>";
		print_r($string);
		print "</pre>";
	}
	
	//Callback to update the comments from the FriendFeed API
	function update_friendfeedcomments() {
		$friendfeed = new FriendFeed('dalziel');
		$options = get_option($this->options_key);
		$ffposts = $friendfeed->fetch_user_feed($options['nickname'], $options['servicename']);
		
		if ($ffposts)
		{
			//Load up the posts from FriendFeed up to the max number of posts to load set
			for ($i = $this->page_size ; $i <= $this->max_posts_to_load - $this->page_size; $i = $i + $this->page_size) {
				$temp_posts = $friendfeed->fetch_user_feed($options['nickname'], $options['servicename'], $i, $this->page_size);
				
				if ($temp_posts && is_array($temp_posts->entries)) {
					$ffposts->entries = array_merge($ffposts->entries, $temp_posts->entries);
				} else {
					$i = $this->max_posts_to_load; 
				}
			}
			
			if ($this->is_debug()) {
				$this->printr(count($ffposts->entries) . " entries loaded from FriendFeed");
			}
		
			$ffdiscussions = get_option($this->discussions_key);
			$blog_url = parse_url(get_option('siteurl'));
			
			$match_count = 0;
			foreach($ffposts->entries as $post)
			{
				$post_url = parse_url($post->service->profileUrl);
				$is_same_blog = ($blog_url['host'] == $post_url['host']) && (
					$blog_url['path'] == '/' || 
					$post_url['path'] == '/' || 
					$post_url['path'] == $blog_url['path']
				); //($post->comments || $post->likes) &&  
				if ($is_same_blog)
				{
					$title = $this->format_title($post->title);
					$ffdiscussions[$title]['id'] = $post->id;
					$ffdiscussions[$title]['link'] = $post->link;
					$ffdiscussions[$title]['comments'] = $post->comments;
					$ffdiscussions[$title]['likes'] = $post->likes;
					$match_count++;
				}
			}	
			update_option($this->discussions_key, $ffdiscussions);
		
			if ($this->is_debug()) {
				$this->printr("$match_count entries loaded, " . count($ffdiscussions) . " total entries stored.");
				$this->printr($ffdiscussions);
				$this->printr($ffposts);
			}

		}	
	}
	
	function add_comment($user, $apikey, $comment, $post_id) {
	
	}
	
	//Add the needed items to the page header
	function add_page_header() {
		$options = get_option($this->options_key);
		
		if ($options['usestyles']) {
			print "\t<link rel=\"stylesheet\" type=\"text/css\" href=\"/" . PLUGINDIR . "/friendfeed-comments/defaultStyle.css\" />\n";
		}
		
		if ($options['showform']) {
			 wp_enqueue_script('prototype');
		}
	}
	
	//Function to render the options page
	function options_page() {
		$options = get_option($this->options_key);
		if ($_POST["wp_friendfeed"])
		{
			if ($_POST['ff_showdebug']) {
				$this->is_debug = true;
			}
		
			if ($_POST['Clear']) {
				delete_option($this->discussions_key);
				$this->update_friendfeedcomments();
				$message = "<strong>Discussions reloaded.</strong>";
			} elseif ($_POST['Delete']) { 
				delete_option($this->discussions_key);
				delete_option($this->options_key);
				$_POST['ff_nickname'] = '';
				$_POST['ff_servicename'] = '';
				$_POST['ff_showcomments'] = '';
				$_POST['ff_headline'] = '';
				$_POST['ff_showform'] = '';				
				$_POST['ff_usestyles'] = '';				
				$message = "<strong>All information related to this plugin has been deleted.</strong> To finish removing this plugin, deactivate it on the Plugins page";
			} else {
				$options['nickname'] = $_POST['ff_nickname'];
				$options['servicename'] = $_POST['ff_servicename'];
				$options['showcomments'] = $_POST['ff_showcomments'];
				$options['headline'] = $_POST['ff_headline'];
				$options['showform'] = $_POST['ff_showform'];
				$options['usestyles'] = $_POST['ff_usestyles'];
				update_option($this->options_key, $options);
				$this->update_friendfeedcomments();
				$message = "<strong>Settings Saved.</strong>  Don't forget to put <code>&lt;?php wp_ffcomments(); ?&gt;</code> on your single post template to display the FriendFeed comments &amp; likes.";
			}
?>
	<div id="message" class="updated fade" style="background-color: rgb(255, 251, 204);"><p><?=$message?></p></div>
<?php
		}
?>
	<div class="wrap">
		<form method="post">
			<input type="hidden" value="true" name="wp_friendfeed"/>
			<h2>FriendFeed Comments</h2>
			<table class="form-table">
			<tbody>
				<tr valign="top">
					<th scope="row">FriendFeed Nickname</th>
					<td><input type="text" size="40" value="<?=$options['nickname']?>" id="ff_nickname" name="ff_nickname" /></td>
				</tr>
				<tr valign="top">
					<th scope="row">Use default stylesheet</th>
					<td>
						<input type="checkbox" name="ff_usestyles" id="ff_usestyles" value="1"<?=($options['usestyles'] ? ' checked="checked"' : '')?> />
						<br />If this is checked the plugin will import the default stylesheet for styling the display of the comments &amp; likes
						lists.  This is meant as a starting point, please feel free to use the styles &amp; adapt them for your site.<br />
						<a href="/<?=PLUGINDIR?>/friendfeed-comments/defaultStyle.css" target="_BLANK">Click here to view the default stylesheet</a>.
					</td>
				</tr>		
				<tr valign="top">
					<th scope="row">Enable comments &amp; likes form</th>
					<td>
						<input type="checkbox" name="ff_showform" id="ff_showform" value="1"<?=($options['showform'] ? ' checked="checked"' : '')?> />
						<br />If this is checked the plugin will display a small form under the comments to allow the reader to add a comment or 
						'like' on the post FriendFeed.  
					</td>
				</tr>					
				<tr valign="top">
					<th scope="row">Headline Text</th>
					<td>
						<input type="text" size="100" value="<?=$options['headline']?>" id="ff_headline" name="ff_headline" />
						<br />This is the text that will be shown above the comments & likes. The tokens {comments} &amp; {likes} will
						be replaced with the number of comments &amp; likes for the post.  Leave this blank to use the default 
						"<em>On FriendFeed, this post was liked by x people and commented on x times</em>".
					</td>
				</tr>					
				<tr valign="top">
					<th scope="row">Show comments by default</th>
					<td>
						<input type="checkbox" name="ff_showcomments" id="ff_showcomments" value="1"<?=($options['showcomments'] ? ' checked="checked"' : '')?> />
						<br />If this is checked the FriendFeed comments &amp; likes will be shown by default.  If not checked the user will have to click on the 'show'
						link to display them.
					</td>
				</tr>	
				<tr valign="top">
					<th scope="row">FriendFeed Servicename</th>
					<td>
						<input type="text" size="40" value="<?=$options['servicename']?>" id="ff_servicename" name="ff_servicename" />
						<br />This is the name that FriendFeed knows your blog as.  Unless you don't use the 'blog' service on FriendFeed, you probably won't need to change this.
					</td>
				</tr>
				<tr valign="top">
					<th scope="row">Display debug information</th>
					<td>
						<input type="checkbox" name="ff_showdebug" id="ff_showdebug" value="1" />
						<br />Check this if you want to see the information that is being loaded by the plugin when you save.  Only needed for troubleshooting.
					</td>
				</tr>
			</tbody>
			</table>
			<p class="submit">
				<input type="submit" value="Save Changes" name="Submit"/>
				<input type="submit" style="color:#DD0000; font-weight:bold;" value="Reset comments/likes" name="Clear" onclick="return confirm('This will delete then reload all FriendFeed comments & likes.  Are you sure?');" />
				<input type="submit" style="color:#DD0000; font-weight:bold;" value="Delete plugin" name="Delete" onclick="return confirm('This will delete all information relating to this plugin.  Only do this if you're intending to remove this plugin.  Are you sure?');" />
			</p>
		</form>
		<div style="background-color:rgb(238, 238, 238); border: 1px solid rgb(85, 85, 85); padding: 5px; margin-top:10px;">
			<p>Did you find this plugin useful?  Please consider donating to help me continue developing it and other plugins.</p>
			<form action="https://www.paypal.com/cgi-bin/webscr" method="post"><input type="hidden" name="cmd" value="_xclick" /><input type="hidden" name="business" value="paypal@slaven.net.au" /><input type="hidden" name="item_name" value="FriendFeed Comments Wordpress Plugin" /><input type="hidden" name="no_note" value="1" /><input type="hidden" name="currency_code" value="AUD" /><input type="hidden" name="tax" value="0" /><input type="hidden" name="bn" value="PP-DonationsBF" /><input type="image" src="https://www.paypal.com/en_US/i/btn/x-click-but04.gif" border="0" name="submit" alt="Make payments with PayPal - it's fast, free and secure!" /></form>
		</div>
	</div>
<?php
	}
	
	function render_likes_list($likes) {
		$output_string = '';
		
		$count = count($likes);
		if ($count) {
			$output_string .= "Liked by\n\t\t\t<ul>\n";
			for($i = 0; $i < $count; $i++) {
				$like = $likes[$i];
				$output_string .= "\t\t\t\t<li><a href=\"{$like->user->profileUrl}\">{$like->user->name}</a>" . ($i < $count - 1 ? ", " : '') . "</li>\n";
			}
			$output_string .= "\t\t\t</ul>\n";
		}
		return $output_string;
	}
	
	//Build up the string to display the list of comments
	function render_comments_list($comments) {
		$output_string = '';
		
		$output_string .= "<ul>\n";
		foreach($comments as $comment) {
			$output_string .= $this->render_comment($comment);
		}
		$output_string .= "</ul>\n";
		
		return $output_string;
	}
	
	function render_comment($comment) {
		$output_string = '';
		$timeformat = get_option('time_format');
		$dateformat = get_option('date_format');		
		$time = date($timeformat, $comment->date);
		$date = date($dateformat, $comment->date);
		$output_string .= "\t\t\t<li class=\"friendfeedcomment\">\n";
		$output_string .= "\t\t\t\t<a href=\"{$comment->user->profileUrl}\"><img style=\"float:left; margin: 0 5px 5px 0px;\" src=\"http://friendfeed.com/{$comment->user->nickname}/picture?size=medium\" alt=\"\" /></a>";
		$output_string .= "\t\t\t\t<div class=\"friendfeedmeta\"><em>{$date} at {$time}</em> <cite><a href=\"{$comment->user->profileUrl}\">{$comment->user->name}</a></cite></div>\n";
		$output_string .= "\t\t\t\t<div class=\"friendfeedcommenttext\">" . make_clickable($comment->body) . "</div><div style=\"clear:both;\"></div>\n";
		$output_string .= "\t\t\t</li>\n";	
		
		return $output_string;
	}
	
	//Get an entry from the list by it's entry id
	function get_discussion_by_id($entryid) {
		$ffdiscussions = get_option($this->discussions_key);
		
		foreach ($ffdiscussions as $discussion) {
			if ($discussion['id'] == $entryid) {
				return $discussion;
			}
		}
		
		return null;
	}
	
	function display_comments() {
		$ffdiscussions = get_option($this->discussions_key);
		$title = $this->format_title(the_title('','',false));
		$options = get_option($this->options_key);
		if ($this->is_debug()) {
			$this->printr($options);
			$this->printr($ffdiscussions);
			$this->printr($title);
		}
		
		if ($ffdiscussions && $ffdiscussions[$title]) {
			$discussion = $ffdiscussions[$title];
?>
	<div id="friendfeedcomments">
		<div id="friendfeedcommentslink"><img src="/<?=PLUGINDIR?>/friendfeed-comments/friendfeedicon.png" alt="" id="friendfeedicon" /><?php
			
			$comment_count = (is_array($discussion['comments']) ? count($discussion['comments']) : 0);
			$like_count = (is_array($discussion['likes']) ? count($discussion['likes']) : 0);
			
			if ($options['headline']) {
				$headline_text = str_replace('{comments}', $comment_count, $options['headline']);
				$headline_text = str_replace('{likes}', $like_count, $headline_text);
			} else {
				$headline_text = 'On FriendFeed, this post was ';
				$headline_text .= "liked by $like_count " . ($like_count <> 1 ? 'people' : 'person');
				$headline_text .= " and ";
				$headline_text .= " commented on $comment_count" . ($comment_count <> 1 ? ' times' : ' time');						
			}
			
			print $headline_text;
?> 	<a href="#" id="togglefriendfeedcommentslink" onclick="return false;" onfocus="blur();"><?=($options['showcomments'] ? 'hide' : 'show')?></a></div>
		<div id="friendfeeddiscussions" style="display: <?=($options['showcomments'] ? 'block' : 'none')?>">
			<div class="friendfeedpermalink"><a href="http://www.friendfeed.com/e/<?=$discussion['id']?>">View this post on FriendFeed</a></div>
<?php
			print "\t\t<div id=\"friendfeedlikeslist\">";
			if ($discussion['likes']) {
				print $this->render_likes_list($discussion['likes']);
			}
			print "</div>\n";
			
			print "\t\t<div id=\"friendfeedcommentslist\">\n\t\t";
			if ($discussion['comments']) {
				print $this->render_comments_list($discussion['comments']);
			}	
			print "</div>\n";
			
			if ($options['showform']) {
?>
			<form id="ff_commentsform" method="post" action="">
			<h3>Add a comment on FriendFeed</h3>
			<textarea id="ff_newcomment" name="ff_newcomment"></textarea><br />
			<label for="ff_username">Username:</label><input type="text" name="ff_username" id="ff_username" value="" /><br />
			<label for="ff_apikey">API Key<sup><a href="https://friendfeed.com/account/api" title="Get your API key from here" target="_BLANK">?</a></sup>:</label><input type="text" name="ff_apikey" id="ff_apikey" value="" /><br />
			<input id="ff_submitcomment" name="ff_submitcomment" value="Add comment" type="submit" /> <input type="button" id="ff_likeentry" name="ff_likeentry" value="Like this post" /> <img src="/<?=PLUGINDIR?>/friendfeed-comments/loading.gif" alt="" id="ff_loadingimage" style="display:none;" />
			<input type="hidden" name="ff_entryid" value="<?=$discussion['id']?>" /> 
			</form>
			
			<script type="text/javascript">
			function addFriendFeedComment(e) {
				if ($('ff_newcomment').getValue()) {
					body = Form.serialize($("ff_commentsform"));
					body += '&ac=comment';
					Form.disable($("ff_commentsform"));
					

					 // submit the form using Ajax
					 new Ajax.Request("/<?=PLUGINDIR?>/friendfeed-comments/friendfeed_ajax.php", {
					   method : 'get',
					   parameters : body,
					   postBody : body,
					   onUninitialized : function() { $('ff_loadingimage').show(); },
					   onSuccess : function(resp) { 
										Element.update('friendfeedcommentslist', resp.responseText); 
										$('ff_newcomment').clear();
									},
					   onFailure : function() { alert("Sorry, the username & API key don't match"); },
					   onComplete : function() { Form.enable($("ff_commentsform")); $('ff_loadingimage').hide(); }
					 });
				}
				Event.stop(e);
			} 

			function AddFriendFeedLike() {
				body = Form.serialize($("ff_commentsform"));
				body += '&ac=like';
				Form.disable($("ff_commentsform"));

				 // submit the form using Ajax
				 new Ajax.Request("/<?=PLUGINDIR?>/friendfeed-comments/friendfeed_ajax.php", {
				   method : 'get',
				   parameters : body,
				   postBody : body,
				   onUninitialized : function() { $('ff_loadingimage').show(); },
				   onSuccess : function(resp) { 
									Element.update('friendfeedlikeslist', resp.responseText); 
									$('ff_newcomment').clear();
								},
				   onFailure : function() { alert("Sorry, the username & API key don't match"); },
				   onComplete : function() { Form.enable($("ff_commentsform")); $('ff_loadingimage').hide(); }
				 });		 
				 Event.stop(e);
			}
			
			Event.observe($("ff_commentsform"), "submit", addFriendFeedComment, false);
			Event.observe($('togglefriendfeedcommentslink'), 'click', function() { $('friendfeeddiscussions').toggle(); $('togglefriendfeedcommentslink').update(($('friendfeeddiscussions').getStyle('display') == 'none' ? 'show' : 'hide'));  });
			Event.observe($('ff_likeentry'), 'click', AddFriendFeedLike, false);
			</script>
<?php				
			}
?>		
		</div>
	</div>
<?php
		}
	}
}

$FriendFeedPostComments = new FriendFeedPostComments();

function wp_ffcomments()
{
	global $FriendFeedPostComments;
	$FriendFeedPostComments->display_comments();
}
?>